	PROGRAM PROFASN
C
C	MAKES MECHANISM ASSIGNMENTS FOR PROFILES
C	Profiles must be processed by ProfPro and data given as compounds
C	in .EMC files.
C	Note: This version outputs GSPRO files in CSV format
C
C	Following special "compound" names used:
C	    2	NONVOL ... Unknown nonvolatile mixture (no MWt) (MS=NVOL)
C	    3	NOASN  ... Unknown volatile mixture (no MWt) (MS=UNKN)
C	    3	UNKNOWN .. Unknown volatile mixture (no MWt) (MS=UNKN)
C	    4	INERT  ... Unspeciated inert compounds (no MWt) (MS=NROG)
C	Following special model species names used.  All output as mass (mwt=1)
C	    1	NOASN  ... Compound with no model species assignment.
C		           MWt info available but not used.
C	    2	NVOL   ... Non-volatile compounds. MWt information available
C		           if nonvolatile compound, but merged with NONVOL
C	    3	UNKN   ... Unknown compounds.
C	    4	NROG   ... Inert mass
C	
C	Use: ProfAsn InFile MechID OutFile, where
C	
C	InFile      ... Full path name of primary input file, which is either:
C	  InFile.SDB  ... Profile data processed by ProfProM containing
C	                  compositions in terms of compounds in SpecDB
C	  anything else . Parameter input file (see below)
C	MechID      ... ID of the mechanism.
C	OutFile     ... Output file name (w/o extension).
C	
C	Program parameters in the input file (InFile) can be:
C	
C	IN=ProfCompFile SARid  ... Full path name of the input file and the
C	                   categorization system used (SARid) (e.g., S40 or CA).
C	                   More than one can be used to process multiple files.
C	UNKN UnknFile  ... Full path name of a file giving compounds in a mixture
C	                   to assign model species to unknokwn mixtures (optional)
C	
C	NOASN NoAsnFile .. Full path name of a file giving compounds in the mixture to
C	                   assign model species to unassigned compound (optional)
C	
C	Output files are:
C	OutFile.GSP    ... Assignments of model species to profiles (GSPRO file)
C	OutFile.CSV    ... Listing of profiles processed and summary information
C
C
C	Calls STSUBS subroutines
C
	CHARACTER EMCUIC*40,OUTFIL*60,OUTFL2*60,INFILE*60,ASNFIL*60
	CHARACTER UNKFIL*60,NOAFIL*60,SDBFIL*60
	CHARACTER MECHID*16,PROFID*16,NAMEIN*16,INLIN*250
	LOGICAL ERRIND
C
C	Model species assignments
	PARAMETER (MAXMS=1200)
	INTEGER NMS,INDNV,INDNOA,INDUNK,INDNR
	CHARACTER MSNAM(MAXMS)*16,NVOLID*16,NOASID*16,UNKID*16,NRID*16
	INTEGER MSLEN(MAXMS)
	CHARACTER ASNIND*1
	REAL MSMWT(MAXMS),MWI
	DATA NVOLID/'NVOL            '/,NOASID/'NOASN           '/
	DATA UNKID/'UNKN            '/,NRID/'NROG            '/
	INTEGER NVLN/4/,NOALN/5/,UNKLN/4/,NRLN/4/
C
C	Compound assignments to model species
	PARAMETER (MAXSPC=2000,MAXASN=5000,MAXMSI=30)
	CHARACTER SPCNAM(MAXSPC)*16,SPCNMI*16,ASNID*1,MSINAM(MAXMSI)*16
	INTEGER NSPC,LASN,ASNLC1(MAXSPC),ASNLC2(MAXSPC),SPCIND(MAXSPC)
	INTEGER ASNMS(MAXASN)
	REAL SPCMW(MAXSPC),ASNWT(MAXASN),ASNIN(MAXMSI),ASNMOL(MAXASN)
	CHARACTER ASNNAM(MAXMSI)*16
	LOGICAL SPCNOA(MAXSPC),SPCNV(MAXSPC),SPCUNK(MAXSPC),SPCNR(MAXSPC)
	LOGICAL ISXMT(MAXSPC)
C
C	Profile assignments
	INTEGER NASN,PROMS(MAXMS)
	REAL PROMOL(MAXMS),PROMAS(MAXMS)
	CHARACTER UNKPRO*16,NOAPRO*16,MSG*16
	INTEGER HASUNK/0/,HASNOA/0/,NASNUN,NASNNO
	INTEGER UNKMS(MAXMS),NOAMS(MAXMS)
	REAL UNKMOL(MAXMS),UNKMAS(MAXMS),NOAMOL(MAXMS),NOAMAS(MAXMS)
	REAL UNKNV,NOANV,UNKNR,NOANR,MOLNOA
	REAL TOTMAS,TOTNV,TOTNOA,TOTNR,TOTUNK,TOTASN
C
	LOGICAL EMCIN,FILEIN,MULTI,EOF,CHANGE
	CHARACTER OUTBUF1*21,OUTBUF2*10,OUTBUF3*5,OUTBUF4*9
	DATA OUTBUF3/',1.0,'/
C
C	IF 'GSPUNK' IS TRUE, THEN THE UNKN AND NOASN PROFILES ARE INCLUDED
C	IN THE GSPRO FILE.  CHANGE THE DATA STATEMENT BELOW TO CONTROL THIS.
C
	LOGICAL GSPUNK/.TRUE./,GSPOUT
C
C	****
C	READ PARAMETERS FROM COMMAD LINE
C
	CALL GETARG (1,INFILE)			! GET COMMAND LINE
	IF (INFILE(1:1).EQ.' ') THEN
		WRITE (6,9999)
		CALL EXIT (2)
	ENDIF

 9999	FORMAT(
     &  /'Use: ProfAsn InFile MechID OutFile, where'
     &  /
     &  /'InFile      ... Full path name of primary input file, which '
     &    ,'is either:'
     &  /'  InFile.SDB  ... Profile data processed by ProfProM contain'
     &    ,'ing'
     &  /'                  compositions in terms of compounds in Spec'
     &    ,'DB'
     &  /'  anything else . Parameter input file (see below)'
     &  /'MechID      ... ID of the mechanism.'
     &  /'OutFile     ... Output file name (w/o extension).'
     &  /
     &  /'Program parameters in the input file (InFile) can be:'
     &  /
     &  /'IN=ProfCompFile SARid  ... Full path name of the input file '
     &    ,'and the'
     &  /'                   categorization system used (SARid) (e.g.,'
     &    ,' S40 or CA).'
     &  /'                   More than one can be used to process mult'
     &    ,'iple files.'
     &  /'UNKN UnknFile  ... Full path name of a file giving compounds'
     &    ,' in a mixture'
     &  /'                   to assign model species to unknokwn mixtu'
     &    ,'res (optional)'
     &  /
     &  /'NOASN NoAsnFile .. Full path name of a file giving compounds'
     &    ,' in the mixture to'
     &  /'                   assign model species to unassigned compou'
     &    ,'nd (optional)'
     &  /
     &  /'Output files are:'
     &  /'OutFile.GSP    ... Assignments of model species to profiles '
     &    ,'(GSPRO file)'
     &  /'OutFile.CSV    ... Listing of profiles processed and summary'
     &    ,' information'
     &  /'ProfCompFile.SDB . Profiles in terms of compounds (if SARid '
     &    ,'used)')
C
	LINFIL=LNBLNK(INFILE)
C
	CALL GETARG(2,MECHID)
	IF (MECHID(1:1).EQ.' ') THEN
		WRITE (6,*) 'Mechanism ID not specified'
		CALL EXIT (2)
	ENDIF
	LMECHID=LNBLNK(MECHID)
	CALL GETARG (3,OUTFIL)
	IF (OUTFIL(1:1).EQ.' ') THEN
		WRITE (6,*) 'GSPRO output file not specified'
		CALL EXIT (2)
	ENDIF
	LOUTFL=LNBLNK(OUTFIL)
	IF (OUTFIL(LOUTFL-3:LOUTFL).NE.'.GSP') THEN
		OUTFIL=OUTFIL(1:LOUTFL)//'.GSP'
		OUTFL2=OUTFIL(1:LOUTFL)//'.CSV'
		LOUTFL=LOUTFL+4
	ELSE
		OUTFL2=OUTFIL(1:LOUTFL-3)//"CSV"
	ENDIF
C
C
C	****
C	READ MODEL SPECIES NAMES AND MWT'S FROM
C	[mechid]MS.DAT
C
	ASNFIL=MECHID(1:LMECHID)//'MS.DAT'
	LASNFL=LMECHID+6
	OPEN (UNIT=1,FILE=ASNFIL,STATUS='OLD',ACTION='READ',ERR=21)
	GOTO 22
   21	WRITE (6,*) 'Model species file = '//ASNFIL(1:LASNFL)
     &	 //' does not exist.'
	CALL EXIT (1)
   22	CONTINUE
C
	NMS=0
   25	READ (1,1,END=28) INLIN
	IF (INLIN.NE.'.') GOTO 25
C
	INDNV=0
	INDNOA=0
	INDUNK=0
	INDNR=0
C
   26	READ (1,1,END=28) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 26
	NAMEIN=INLIN(1:16)
	READ (INLIN,'(19X,F7.2)',ERR=27) MWI
	IF (NMS.EQ.MAXMS) THEN
		CLOSE (UNIT=1)
		GOTO 271
	ELSEIF (MWI.LE.0.0) THEN
		WRITE (6,*) 'Illegal MWT for '//NAMEIN
		GOTO 27
	ENDIF
	NMS=NMS+1
	MSNAM(NMS)=NAMEIN
	MSLEN(NMS)=LNBLNK(NAMEIN)
	IF (NAMEIN.EQ.NVOLID) THEN
		INDNV=NMS
		MWI=1.0
	ELSEIF (NAMEIN.EQ.NOASID) THEN
		INDNOA=NMS
		MWI=1.0
	ELSEIF (NAMEIN.EQ.UNKID) THEN
		INDUNK=NMS
		MWI=1.0
	ELSEIF (NAMEIN.EQ.NRID) THEN
		INDNR=NMS
		MWI=1.0
	ENDIF
	MSMWT(NMS)=MWI
	GOTO 26
C
   27	WRITE (6,*) 'Error reading data from model species file = '
     &	 //ASNFIL(1:LASNFL)
	WRITE (6,*) INLIN(1:LNBLNK(INLIN))
	CLOSE (UNIT=1)
	CALL EXIT (1)
C
  271	WRITE (6,*) 'Too many model species.  Increase MAXMS'
	CALL EXIT (1)
C
   28	CLOSE (UNIT=2)
	IF (NMS.EQ.0) THEN
		WRITE (6,*) 'No model species listed in file = '
     &		 //ASNFIL(1:LASNFL)
		CLOSE (UNIT=1)
		CALL EXIT (1)
	ENDIF
	IF (INDNV.EQ.0) THEN
		IF (NMS.EQ.MAXMS) GOTO 271
		NMS=NMS+1
		INDNV=NMS
		MSNAM(NMS)=NVOLID
		MSMWT(NMS)=1.0
		MSLEN(NMS)=NVLN
	ENDIF
	IF (INDNOA.EQ.0) THEN
		IF (NMS.EQ.MAXMS) GOTO 271
		NMS=NMS+1
		INDNOA=NMS
		MSNAM(NMS)=NOASID
		MSMWT(NMS)=1.0
		MSLEN(NMS)=NOALN
	ENDIF
	IF (INDUNK.EQ.0) THEN
		IF (NMS.EQ.MAXMS) GOTO 271
		NMS=NMS+1
		INDUNK=NMS
		MSNAM(NMS)=UNKID
		MSMWT(NMS)=1.0
		MSLEN(NMS)=UNKLN
	ENDIF
	IF (INDNR.EQ.0) THEN
		IF (NMS.EQ.MAXMS) GOTO 271
		NMS=NMS+1
		INDNR=NMS
		MSNAM(NMS)=NRID
		MSMWT(NMS)=1.0
		MSLEN(NMS)=NRLN
	ENDIF
C
C
C	****
C	READ MODEL SPECIES ASSIGNMENT DATA FROM [mechid]ASN.DAT
C
	ASNFIL=MECHID(1:LMECHID)//'ASN.DAT'
	LASNFL=LMECHID+7
	OPEN (UNIT=1,FILE=ASNFIL,STATUS='OLD',ACTION='READ',ERR=2)
	GOTO 3
    2	WRITE (6,*) 'Mechanism assign file = '//ASNFIL(1:LASNFL)
     &	 //' does not exist.'
	CALL EXIT (1)
C
    3	CONTINUE
C
C	PROCESS HEADERS DEPENDING ON FILE
C
	INDFMT=0
	NMSI=0
   10	READ (1,1,END=12) INLIN
    1	FORMAT (A)
	IF (INLIN(1:8).EQ.'FORMAT=1') THEN
		INDFMT=1
		NCOL=1
	ELSEIF (INLIN(1:8).EQ.'FORMAT=2') THEN
		INDFMT=2
		IF (INLIN(9:10).EQ.'  ') THEN
			NCOL=1
		ELSE
			READ (INLIN(10:10),'(I1)',ERR=12) NCOL
			IF (NCOL.EQ.0) THEN NCOL=1
		ENDIF
	ELSEIF (INLIN(1:8).EQ.'FORMAT=3') THEN
		INDFMT=3
	ELSEIF (INLIN(1:8).EQ.'NUMSPEC=') THEN
		L=LNBLNK(INLIN)
		IF (L.EQ.9) THEN
			READ (INLIN,'(8X,I1)',ERR=12) NMSI
		ELSEIF (L.EQ.10) THEN
			READ (INLIN,'(8X,I2)',ERR=12) NMSI
			IF (NMSI.GT.MAXMSI) THEN
				Write (6,*) 'Too many Format=3 model '
     &				 //'species.  Increase MAXMSI'
				GOTO 12
			ENDIF
		ELSE
			GOTO 12
		ENDIF
		IF (NMSI.GT.NMS.OR.NMSI.EQ.0) THEN
			WRITE (6,*) 'NMSI,NMS=',NMSI,NMS
			GOTO 121
		ENDIF
	ELSEIF (INLIN(1:1).EQ.'.') THEN
		IF (INDFMT.EQ.0) THEN
			WRITE (6,*) 'No FORMAT= input.'
			GOTO 12
		ELSEIF ((INDFMT.EQ.3).AND.(NMSI.EQ.0)) THEN
			WRITE (6,*) 'NUMSPEC not given for FORMAT=3'
			GOTO 12
		ENDIF
		GOTO 20
	ENDIF
	GOTO 10
   12	WRITE (6,*) 'Mechanism assign file = '//ASNFIL(1:LASNFL)
     &	 //' has no assignments or bad header data.'
	GOTO 122
C
  121	WRITE (6,*) 'Model species listed in '//MECHID(1:LMECHID)
     &	 //'ASN.DAT and '//MECHID(1:LMECHID)//'MS.DAT not consistent'
C
  122	CLOSE (UNIT=1)
	CALL EXIT (1)
C
   20	IF (INDFMT.EQ.3) THEN
		READ (1,'(36X,30A7)') (MSINAM(I),I=1,NMSI)
		DO 281 I=1,NMSI
		IF (MSINAM(I).NE.MSNAM(I)) THEN
			WRITE (6,*) 'MS,Asn=',MSNAM(I),', ',MSINAM(I)
			GOTO 121
		ENDIF
  281		CONTINUE
		IF (NMS.GT.NMSI) THEN
			DO 282 I=NMSI+1,NMS
  282			ASNIN(I)=0.0
		ENDIF
	ENDIF
C
C	HEADERS READ.  READ ASSIGNMENTS FOR INDIVIDUAL SPECIES
C	SPCIND CODE:
C		0 = Assigned a normal model species (mole basis)
C		1 = NOAS = compound with no MS assignment (mass basis)
C		           (Set to -1 if NOASN used for MS assigns)
C		2 = NVOL = Nonvolatile mass
C		3 = UNKN = Unknown mass
C		4 = NROG = Inert mass
	NSPC=0
	NASN=0
   30	READ (1,1,END=40) INLIN
	IF (INDFMT.EQ.1) THEN
		READ (INLIN,'(A16,F8.3,6X,I1,2X,A16)',ERR=39) 
     &		 SPCNMI,SPCMWI,ISVOC,ASNNAM(1)
		ASNIN(1)=1.0
	ELSEIF (INDFMT.EQ.2) THEN
		READ (INLIN,'(A16,F8.3,6X,I1,2X,10(A16,F7.3,1X))'
     &		 ,ERR=39) SPCNMI,SPCMWI,ISVOC,(ASNNAM(I),ASNIN(I),
     &		 I=1,NCOL)
	ELSE
		READ (INLIN,'(A16,F8.3,5X,A1,3X,I1,1X,30F7.2)',ERR=39)
     &		 SPCNMI,SPCMWI,ASNIND,ISVOC,(ASNIN(I),I=1,NMSI)
	ENDIF
	IF (NSPC.EQ.MAXSPC) THEN
		WRITE (6,*) 'Too many compounds.  Increase MAXSPC'
		CLOSE (UNIT=1)
		CALL EXIT (2)
	ENDIF
	NSPC=NSPC+1
	SPCNAM(NSPC)=SPCNMI
	SPCMW(NSPC)=SPCMWI
	ISXMT(NSPC)=ISVOC.EQ.0
	IF (SPCNMI.EQ.'NOASN           ') THEN
		SPCIND(NSPC)=3
	ELSEIF (SPCNMI.EQ.'UNKNOWN         ') THEN
		SPCIND(NSPC)=3
	ELSEIF (SPCNMI.EQ.'NONVOL          ') THEN
		SPCIND(NSPC)=2
	ELSEIF (SPCNMI.EQ.'INERT           ') THEN
		SPCIND(NSPC)=4
	ELSEIF (INDFMT.LE.2) THEN
		NAMEIN=ASNNAM(1)
		IF (NAMEIN.EQ.NRID) THEN
			SPCIND(NSPC)=4
		ELSEIF (NAMEIN.EQ.NVOLID) THEN
			SPCIND(NSPC)=2
		ELSEIF (NAMEIN.EQ.NOASID) THEN
			SPCIND(NSPC)=1
			SPCMW(NSPC)=SPCMWI
		ELSEIF (NAMEIN.EQ.UNKID) THEN
			SPCIND(NSPC)=3
		ELSE
			SPCIND(NSPC)=0
			ASNLC1(NSPC)=NASN+1
			TOTWTI=0.0
			DO 33 ICOL=1,NCOL
			NAMEIN=ASNNAM(ICOL)
			IF (NAMEIN(1:1).NE.' ') THEN
				DO 32 I=1,NMS
				IF (NAMEIN.EQ.MSNAM(I)) THEN
					NASN=NASN+1
					IF (NASN.GT.MAXASN) STOP
     &					 'INCREASE MAXASN'
					ASNMS(NASN)=I
					ASNMOL(NASN)=ASNIN(ICOL)
					ASNWT(NASN)=ASNIN(ICOL)*MSMWT(I)
					TOTWTI=TOTWTI+ASNWT(NASN)
					GOTO 33
				ENDIF
   32				CONTINUE
				WRITE (6,*) 'Unrecognized model sp'//
     &				 NAMEIN//'ecies for species '//SPCNMI
				CLOSE (UNIT=1)
				CALL EXIT (1)
			ENDIF
   33			CONTINUE
			ASNLC2(NSPC)=NASN
			IF (TOTWTI.NE.0.0) THEN
				DO 3511 I=ASNLC1(NSPC),NASN
 3511				ASNWT(I)=ASNWT(I)/TOTWTI
			ENDIF

		ENDIF
C
	ELSE ! INDFMT=3
		INDI=0
		IF (ASNIND.EQ.'U') THEN
			INDI=1
			SPCMW(NSPC)=SPCMWI
		ELSEIF (ASNIND.EQ.'N') THEN
			INDI=2
		ELSEIF (ASNIN(INDNV).GT.0.0) THEN
			INDI=2
		ELSEIF (ASNIN(INDUNK).GT.0.0) THEN
			INDI=3
		ELSEIF (ASNIN(INDNR).GT.0.0) THEN
			INDI=4
		ENDIF
		IF (INDI.EQ.0) THEN
			ERRIND=.TRUE.
			ASNLC1(NSPC)=NASN+1
			TOTWTI=0.0
			DO 35 I=1,NMSI
			IF (ASNIN(I).NE.0.0) THEN
				ERRIND=.FALSE.
				NASN=NASN+1
				IF (NASN.GT.MAXASN) STOP 'INCREASE MAXASN'
				ASNMS(NASN)=I
				ASNMOL(NASN)=ASNIN(I)
				ASNWT(NASN)=ASNIN(I)*MSMWT(I)
				TOTWTI=TOTWTI+ASNWT(NASN)
			ENDIF
   35			CONTINUE
			IF (ERRIND) THEN
				INDI=1
			ELSE
				INDI=0
				ASNLC2(NSPC)=NASN
				DO 351 I=ASNLC1(NSPC),NASN
  351				ASNWT(I)=ASNWT(I)/TOTWTI
			ENDIF
		ENDIF
		SPCIND(NSPC)=INDI
	ENDIF
	GOTO 30
C
  39	WRITE (6,*) 'Error reading data in file = '//MECHID(1:LMECHID)
     &	 //'ASN.DAT'
	WRITE (6,*) INLIN(1:LNBLNK(INLIN))
	CLOSE (UNIT=1)
	CALL EXIT (1)
C
  40	CLOSE (UNIT=1)
C
C	DONE READING MECHANISM AND SPECIES ASSIGNMENTS
C
C
C	****
C	OPEN OUTPUT FILE FOR STATUS INFORMATION (UNIT=2)
C	OPEN OUTPUT FILE FOR GSPRO OUTPUT (UNIT=3)
C
	OPEN (UNIT=2,FILE=OUTFL2(1:LOUTFL),STATUS='UNKNOWN')
	WRITE (2,1) 'Mech:,'//MECHID(1:LMECHID)
	WRITE (2,1) 'Input:,'//INFILE(1:LINFIL)
	WRITE (2,1) 'Output:,'//OUTFIL(1:LOUTFL)
	WRITE (2,*)
	WRITE (2,1) 'Profile,Msg,InputWt,AsnFrc,UNRfrc,NVfrc,NOAfrc,'
     &	 //'UNKfrc,nVOCfrc,Compounds,Asn''s'
C
	OPEN (UNIT=3,FILE=OUTFIL(1:LOUTFL),STATUS='UNKNOWN')
C
C
C	****
C	OPEN INPUT FILE
C	IF FILE HAS EXTENSION .EMC IT IS ASSUMED TO DEFINE
C	A SINGLE PROFILE.
C
C	MULTI INDICATES WHETHER UNIT 1 IS USED TO GET PROFILE
C	NAMES SO MULTIPLE PROFILES CAN BE PROCESSED.
C
C	INFMT=INPUT FORMAT.  0 = SDB, 1 = EMI, 2 = PROFDB
C
	OPEN (UNIT=1,FILE=INFILE(1:LINFIL),STATUS='OLD',
     &	 ACTION='READ',ERR=80)
	INU=1
C
	IF (INFILE(LINFIL-3:LINFIL).EQ.'.EMC') THEN
		MULTI=.FALSE.
		EMCIN=.TRUE.
		LPROID=LOUTFL-4
		PROFID=OUTFIL(1:LPROID)
	ELSE
		MULTI=.TRUE.
	ENDIF
C
	GOTO 90
C
   80	WRITE (6,*) 'File = '//INFILE(1:LINFIL)//' is empty or '
     &	 //'does not exist'
	CALL EXIT (1)
C
C	****
C	INITIALIZE PROFILE INFO. 
C
   90	NPROF=0
	NCHPRO=0
	UNKPRO=' '
	NOAPRO=' '
	LUNKFL=0
	LNOAFL=0
	IF (.NOT.MULTI) GOTO 102
C
C	****
C	READ NEXT LINE OF MAIN INPUT FILE UNLESS EMC OR EMI INPUT IS USED
C
  100	READ (INU,1,END=900) INLIN
	IF ((INLIN(1:1).EQ.' ').OR.(INLIN(1:1).EQ.'!').OR.
     &	 (INLIN(1:1).EQ.'.')) THEN
		GOTO 100
	ELSEIF (INLIN(1:6).EQ.'INPUT=') THEN
		IF (INU.EQ.4) THEN
			WRITE (6,*) 'Can''t have recursive INPUT''s'
			GOTO 800
		ENDIF
		LIN=LNBLNK(INLIN)
		CALL PARSE0 (3,INLIN(7:LIN),LIN-6)
		CALL PARSE1 (3,SDBFIL,LSDBFL,60,INLIN(7:LIN))
		IF (SDBFIL(1:1).EQ.' ') THEN
			SDBFIL='TMP.SDB'
			LSDBFL=7
C	??	Why doesn't the following compile?
C		ELSEIF (SBDFIL(LSDBFL-3:LSDBFL).NE.'.SDB') THEN 
		ELSE
			SDBFIL=SDBFIL(1:LSDBFL)//'.SDB'
			LSDBFL=LSDBFL+4
		ENDIF
		OPEN (UNIT=4,FILE=SDBFIL,STATUS='OLD',ERR=1014)
			CLOSE (UNIT=4,STATUS='DELETE')
 1014		CONTINUE
C **NOTE**	the following may need to be modified to run on Unix Systems
		CALL SYSTEM ('PROFPRE '//INLIN(7:LIN))
		OPEN (UNIT=4,FILE=SDBFIL,STATUS='OLD',ACTION='READ',
     &		 ERR=1013)
		INU=4
		GOTO 100
 1013		WRITE (6,*) 'Error running ProfPre with '//
     &		 INLIN(7:LNBLNK(INLIN))
		GOTO 100
	ELSEIF (INLIN(1:11).EQ.'MULTI-PROF ') THEN
		GOTO 100
	ENDIF
C
	CALL PARSE0 (2,INLIN,80)
	CALL PARSE1 (1,PROFID,LPROID,16,INLIN)
C
C	OPEN PROFILE DATA INPUT FILE IF APPLICABLE.
C	ASSUMED TO BE IN EMC FORMAT
C
	FILEIN=.FALSE.
	EMCIN=.FALSE.
C
	IF (INU.EQ.1) THEN
		CALL PARSE1 (2,INFILE,LINFIL,60,INLIN)
		IF (INFILE(1:1).NE.' ') THEN
			OPEN (UNIT=4,FILE=INFILE(1:LINFIL),STATUS='OLD',
     &			 ACTION='READ',ERR=1001)
			INU=4
			FILEIN=.TRUE.
			EMCIN=.TRUE.
			GOTO 1002
 1001			WRITE (6,*) 'Error opening file = '//INFILE(1:LINFIL)
			GOTO 800
		ENDIF
	ENDIF
 1002	CONTINUE
C
C	***
C	CONTROL REACHES HERE REGARDLESS IF ONE OR MULTI PROFILE INPUT.
C	READ PAST HEADERS IF EMC INPUT
C
  102	IF (EMCIN) THEN
 1021		READ (INU,1,END=1026) INLIN
		IF (INLIN(1:1).NE.'.') GOTO 1021
	ENDIF
C
	GOTO 1029
 1026		WRITE (6,*) 'No data in '//INFILE(1:LINFIL)
		IF (INU.EQ.4) CLOSE (UNIT=4)
		GOTO 800
 1029	CONTINUE
C
C	****
C	INITILIZE PROFILE
C
  400	TOTMAS=0.0
	TOTMOL=0.0
	TOTNV=0.0
	TOTNOA=0.0
	MOLNOA=0.0
	TOTNR=0.0
	TOTUNK=0.0
	TOTASN=0.0
	TOTXMT=0.0
	NASNP=0
	NCMPD=0
	MSG=',ok'
	LMSG=3
C
	IF (PROFID(1:5).EQ.'UNKN ') THEN
		HASUNK=2
		UNKPRO=PROFID
		UNKFIL=INFILE
		LUNKFL=LINFIL
	ELSEIF (PROFID(1:6).EQ.'NOASN ') THEN
		HASNOA=2
		NOAPRO=PROFID
		NOAFIL=INFILE
		LNOAFL=LINFIL
	ENDIF
C
C	****
C	READ AND PROCESS NEXT SPECIES
C
  110	READ (INU,1,END=200) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') THEN
		GOTO 110
	ELSEIF (INLIN(1:1).EQ.'.') THEN
		GOTO 201
	ENDIF
	CALL PARSE0 (2,INLIN,80)
	CALL PARSE1 (1,SPCNMI,LNAM,16,INLIN)
	IF (SPCNMI(1:1).EQ.' ') GOTO 110
	CALL PARSER (2,SPCWTI,ERRIND,INLIN)
	IF (ERRIND) THEN
		WRITE (6,*) 'Error reading data for profile '//
     &		 PROFID(1:LPROID)//' from '//INFILE(1:LINFIL)
		WRITE (6,*) 'Input='//INLIN(1:LNBLNK(INLIN)),INU
		WRITE (2,1) PROFID(1:LPROID)//',Error'
		IF (INU.EQ.4) CLOSE (UNIT=4)
		GOTO 800
	ENDIF
C
C	PROCESS SPECIES INPUT
C
	TOTMAS=TOTMAS+SPCWTI
	NCMPD=NCMPD+1
	IF (SPCNMI(1:7).EQ.'NONVOL ') THEN
		TOTNV=TOTNV+SPCWTI
	ELSEIF (SPCNMI(1:6).EQ.'NOASN ') THEN
		TOTUNK=TOTUNK+SPCWTI
	ELSEIF (SPCNMI(1:8).EQ.'UNKNOWN ') THEN
		TOTUNK=TOTUNK+SPCWTI
	ELSEIF (SPCNMI(1:6).EQ.'INERT ') THEN
		STOP 'INERT SPECIES NOT SUPPORTED'
	ELSE
		DO 120 ISPC=1,NSPC
		IF (SPCNMI.EQ.SPCNAM(ISPC)) THEN
			IF (ISXMT(ISPC)) TOTXMT=TOTXMT+SPCWTI
			IF (SPCIND(ISPC).EQ.1) THEN
				TOTNOA=TOTNOA+SPCWTI
				MOLNOA=MOLNOA+(SPCWTI/SPCMW(ISPC))
			ELSEIF (SPCIND(ISPC).EQ.2) THEN
				TOTNV=TOTNV+SPCWTI
			ELSEIF (SPCIND(ISPC).EQ.3) THEN
				TOTUNK=TOTUNK+SPCWTI
			ELSEIF (SPCIND(ISPC).EQ.4) THEN
				TOTNR=TOTNR+SPCWTI
				TOTMOL=TOTMOL+(SPCWTI/SPCMW(ISPC))
			ELSE
				TOTASN=TOTASN+SPCWTI
				IF (SPCIND(ISPC).LT.0) THEN
					TOTNOA=TOTNOA-SPCWTI
					MSG=',[d]'
					LMSG=4
				ENDIF
				SPCMOL=SPCWTI/SPCMW(ISPC)
				TOTMOL=TOTMOL+SPCMOL
				DO 130 I=ASNLC1(ISPC),ASNLC2(ISPC)
				IMS=ASNMS(I)
				ASNWTI=SPCWTI*ASNWT(I)
				ASNMLI=SPCMOL*ASNMOL(I)
				DO 132 J=1,NASNP
				IF (PROMS(J).EQ.IMS) THEN
					PROMOL(J)=PROMOL(J)+ASNMLI
					PROMAS(J)=PROMAS(J)+ASNWTI
					GOTO 130
				ENDIF
  132				CONTINUE
				NASNP=NASNP+1
				PROMS(NASNP)=IMS
				PROMOL(NASNP)=ASNMLI
				PROMAS(NASNP)=ASNWTI
  130				CONTINUE
			ENDIF
			GOTO 110
		ENDIF
  120		CONTINUE
		WRITE (6,*) 'Species = '//SPCNMI(1:LNBLNK(SPCNMI))//
     &		 ', in file = '//INFILE(1:LINFIL)//' not found.'
		WRITE (2,1) PROFID(1:LPROID)//',Not found: '//
     &	 	 SPCNMI(1:LNBLNK(SPCNMI))
		IF (INU.EQ.4) CLOSE (UNIT=4)
		GOTO 800
	ENDIF
	GOTO 110
C
C	END OF FILE ON PROFILE INPUT
C
  200	CLOSE (UNIT=INU)
	IF (INU.EQ.4) THEN
		INU=1
		EOF=.FALSE.
	ELSE
		EOF=.TRUE.
	ENDIF		
C
C	FINISH PROCESSING PROFILE
C
  201	IF (TOTMAS.EQ.0.0) THEN
		WRITE (6,*) 'No mass in '//INFILE(1:LINFIL)
		WRITE (2,1) PROFID(1:LPROID)//',No mass'
		GOTO 100
	ELSE
		WRITE (NAMEIN,'(F7.4)') TOTMAS
		IF (NAMEIN.NE.' 1.0000') THEN
			TOTNV=TOTNV/TOTMAS
			TOTNOA=TOTNOA/TOTMAS
			TOTNR=TOTNR/TOTMAS
			TOTUNK=TOTUNK/TOTMAS
			TOTASN=TOTASN/TOTMAS
			DO 202 I=1,NASNP
			PROMAS(I)=PROMAS(I)/TOTMAS
  202			PROMOL(I)=PROMOL(I)/TOTMAS
		ENDIF
	ENDIF
C
C	Summary output
C
	IF (HASUNK.EQ.2) THEN
		MSG=',[a]'
		LMSG=4
		DO 220 I=1,NASNP
		UNKMS(I)=PROMS(I)
		UNKMOL(I)=PROMOL(I)
  220		UNKMAS(I)=PROMAS(I)
		NASNUN=NASNP
		UNKASN=TOTASN
		UNKNV=TOTNV
		UNKNR=TOTNR
	ELSEIF (HASNOA.EQ.2) THEN
		MSG=',[b]'
		LMSG=4
C		DEFINE ASSIGMENTS FOR UNASSIGNED SPECIES
		IF ((TOTNOA+TOTNV+TOTUNK).NE.0.0) WRITE (6,*) 
     &		 'WARNING: NOASN NOA,NV,UNK =',TOTNOA,TOTNV,TOTUNK
		IASN1=0
		DO 221 ISPC=1,NSPC
		IF (SPCIND(ISPC).EQ.1) THEN
			IF (IASN1.EQ.0) THEN
				IASN1=NASN+1
				DO 2211 I=1,NASNP
				NASN=NASN+1
				IF (NASN.GT.MAXASN) STOP 'INCREASE MAXASN'
				ASNMS(NASN)=PROMS(I)
				ASNWT(NASN)=PROMAS(I)
				ASNMOL(NASN)=PROMOL(I)/TOTMOL
 2211				CONTINUE
				IASN2=NASN
			ENDIF
			ASNLC1(ISPC)=IASN1
			ASNLC2(ISPC)=NASN
			SPCIND(ISPC)=-1
		ENDIF
  221		CONTINUE
		HASNOA=1
		GSPOUT=GSPUNK
	ENDIF
C
C	PROCESS UNKNOWN
C
	CHANGE=.FALSE.
	GSPOUT=.TRUE.
C
	IF ((HASUNK.EQ.1).AND.(TOTUNK.NE.0.0)) THEN
C		MERGE UNKNOWNS IF ANY
		DO 301 I=1,NASNUN
		IMS=UNKMS(I)
		DO 302 J=1,NASNP
		IF (PROMS(J).EQ.IMS) THEN
			JX=J
			GOTO 303
		ENDIF
  302		CONTINUE
		NASNP=NASNP+1
		PROMS(NASNP)=IMS
		PROMOL(NASNP)=TOTUNK*UNKMOL(I)
		PROMAS(NASNP)=TOTUNK*UNKMAS(I)
		GOTO 301
  303		PROMOL(JX)=PROMOL(JX)+(TOTUNK*UNKMOL(I))
		PROMAS(JX)=PROMAS(JX)+(TOTUNK*UNKMAS(I))
  301		CONTINUE
		TOTASN=TOTASN+(UNKASN*TOTUNK)
		TOTNV=TOTNV+(UNKNV*TOTUNK)
		TOTNR=TOTNR+(UNKNR*TOTUNK)
		TOTUNK=-TOTUNK		
		CHANGE=.TRUE.
C
	ELSEIF (HASUNK.EQ.2) THEN
		HASUNK=1
		GSPOUT=GSPUNK
	ENDIF
C
	IF (CHANGE) THEN
		NCHPRO=NCHPRO+1
		MSG=',[c]'
		LMSG=4
	ENDIF
C
C	GSPRO output
C
	L1=LNBLNK(PROFID)
	OUTBUF1=PROFID(1:L1)//',TOG,'
	L1=L1+5
	IF (GSPOUT) THEN
		IF (NASNP.GT.0) THEN
			DO 210 I=1,NASNP
			J=PROMS(I)
			WRITE (OUTBUF2,'(1PE10.4)') PROMOL(I)
			WRITE (OUTBUF4,'(F9.7)') PROMAS(I)
  210			WRITE (3,1) OUTBUF1(1:L1)//MSNAM(J)(1:MSLEN(J))//','//
     &			 OUTBUF2//OUTBUF3//OUTBUF4
		ENDIF
C  211		FORMAT (A16,1X,'TOG',2X,A8,1PE12.4,2X,'1.0',1X,0PF10.7)
		IF (TOTNR.NE.0.0) THEN
			WRITE (OUTBUF4,'(F9.7)') TOTNR
			WRITE (3,1) OUTBUF1(1:L1)//NRID(1:NRLN)//','//OUTBUF4
     &			 //OUTBUF3//OUTBUF4
			NASNP=NASNP+1
		ENDIF
		IF (TOTNV.NE.0.0) THEN
			WRITE (OUTBUF4,'(F9.7)') TOTNV
			WRITE (3,1) OUTBUF1(1:L1)//NVOLID(1:NVLN)//','//OUTBUF4
     &			 //OUTBUF3//OUTBUF4
			NASNP=NASNP+1
		ENDIF
		IF (TOTNOA.GT.0.0) THEN
			WRITE (OUTBUF2,'(1PE10.4)') MOLNOA
			WRITE (OUTBUF4,'(F9.7)') TOTNOA
			WRITE (3,1) OUTBUF1(1:L1)//NOASID(1:NOALN)//','//OUTBUF2
     &			 //OUTBUF3//OUTBUF4
			NASNP=NASNP+1
		ENDIF
		IF (TOTUNK.GT.0.0) THEN
			WRITE (OUTBUF4,'(F9.7)') TOTUNK
			WRITE (3,1) OUTBUF1(1:L1)//UNKID(1:UNKLN)//','//OUTBUF4
     &			 //OUTBUF3//OUTBUF4
			NASNP=NASNP+1
		ENDIF
	ENDIF
C
C	SUMMARY OUTPUT
C
	IF (TOTMAS.GT.0.0) TOTXMT=TOTXMT/TOTMAS
	WRITE (2,'(A,7('','',F7.4),2('','',I3))') 
     &	 PROFID(1:LPROID)//MSG(1:LMSG),TOTMAS,TOTASN,TOTNR,TOTNV,
     &	 TOTNOA,TOTUNK,TOTXMT,NCMPD,NASNP
C
	NPROF=NPROF+1
	IF (.NOT.EOF) GOTO 100
C
C	DONE
C
  900	CLOSE (UNIT=1)
	WRITE (6,*) 'Files = '//OUTFIL(1:LOUTFL)
     &	 //' and '//OUTFL2(1:LOUTFL)//' output.'
	WRITE (6,901) ' Profiles output =',NPROF
	IF (HASUNK.EQ.1) THEN
		IF (LUNKFL.GT.0) THEN
			WRITE (2,1) ',[a],Used to assign model '
     &			 //'species to unknown mass. File = '
     &			 //UNKFIL(1:LUNKFL)
		ELSE
			WRITE (2,1) ',[a],Used to assign model '
     &			 //'species to unknown mass'
		ENDIF
	ENDIF
	IF (HASNOA.GT.0) THEN
		IF (LNOAFL.GT.0) THEN
			WRITE (2,1) ',[b],Used to assign model '
     &			 //'species to unassigned moles. File ='
     &			 //NOAFIL(1:LNOAFL)
		ELSE
			WRITE (2,1) ',[b],Used to assign model '
     &			 //'species to unassigned moles'
		ENDIF
	ENDIF
	IF (NCHPRO.NE.0) THEN
		WRITE (2,1) ',[c],Negative UNKfrc indicates that '
     &		 //'model species were assigned to unknown mass'
  901		FORMAT (A,I5)
	ENDIF
	IF (HASNOA.GT.0) WRITE (2,1) ',[d],Negative NOAfrc indicates '
     &		 //'that model species were assigned to unassigned '
     &		 //'moles'
C
	CLOSE (UNIT=2)
	CLOSE (UNIT=3)
	CALL EXIT (0)
C
C	ERROR
C
  800	CLOSE (UNIT=1)
	CLOSE (UNIT=2)
	CLOSE (UNIT=3)
	CALL EXIT (2)
C
	END
