	PROGRAM PROFPRE
C
C	PROFILE PRE-PROCESSOR -- for multiple profiles in one file.
C	Format suitable for being called by ProfAsn
C
C	This version of the program converts profile database files
C	from SAROAD classes to speciation database compound categories.
C	Input and output file must be given on command line
C
C	Use: PROFPPRE InputFile SARtype [OutputFile]
C	
C	Where InputFile is the name of the input file with records of 
C	the type ProfileID SAROAD masss fraction
C
C	SARtype is the SAROAD code used in the database file
C	(e.g, "TX" for Texas SAROAD, "S40" for Speciate 4 categories
C
C	OutputFile is optional and contains the processed profile data.
C	an extension ".SDB" is added if not present.
C	If the output file is not given, output goes to TMP.SDB
C
C	Outputs diagnostic information in ProfProM.csv
C
C	Mixture assignments are read from SPECASN.DAT
C	SAROAD assignments are read from [SARtype]ASN.DAT
C
C	-----------------------------------------------------------
C	Calls STSUBS subroutines
C
C	SAROAD ASSIGNMENT DATA
C
	PARAMETER (MAXSAR=2000)
	CHARACTER SARNAM(MAXSAR)*12,SARID(MAXSAR)*16
	INTEGER NSARID
C
C	ASSIGNMENT OF SPECIES TO UNIQUE ID'S (UID'S)
C
	PARAMETER (MAXSPC=16000,MAXUID=4000,MAXCMP=2000)
	CHARACTER UIDLST(MAXUID)*16,UIDSPC(MAXSPC)*16,CMPID(MAXCMP)*16
	REAL SPCWT(MAXSPC)
	INTEGER NUID1(MAXUID),NUID2(MAXUID)
	INTEGER NUID,NSPCID
C
C	COMPOUND DESCRIPTION INFORMATION
C
C	EMISSIONS FILE DATA
C
	INTEGER NCMP
	REAL TOTWT,CMPWT(MAXCMP)
C
C	LOCAL VARIABLES
C
	CHARACTER SARI*16,UID*16,OLDUID*16,CMPI*16
	CHARACTER PROFID*16,PROFIN*16
	LOGICAL EOF
	CHARACTER SARTYP*4
	CHARACTER INLIN*255,INFILE*60
	CHARACTER OUTFIL*60
	LOGICAL NOSAR,ERRIND
C
C	OPEN INPUT FILE AND READ PARAMETERS
C
	CALL GETARG (1,INFILE)
	CALL GETARG (2,SARTYP)
	CALL GETARG (3,OUTFIL)
	IF (INFILE(1:1).EQ.' ') THEN
		WRITE (6,*) 'Converts a multiple profile database file'
     &		 //' into a file giving profiles in'
		WRITE (6,*) 'terms of compounds in SDB format.  Record'
     &		 //'s in database input file must be'
		WRITE (6,*) '     Profile  SpeciesID  Mass'
		WRITE (6,*)
		WRITE (6,*) 'Use: ProfPre InputFile SARtype [SDBfile]'
		WRITE (6,*)
		WRITE (6,*) 'InputFile = full name of input database fi'
     &		 //'le.'
		WRITE (6,*) 'SARtype   = Species classification in inpu'
     &		 //'t file.'
		WRITE (6,*) '            e.g., EPA, CA, or TX for EPA, '
     &		 //'California, or Texas SAROAD,'
		WRITE (6,*) '            S40 for Speciate 4,or SDB or N'
     &		 //'ONE for Speciation DB format'
		WRITE (6,*) 'SDBfile   = Name of output file.  Extensio'
     &		 //'n ".SDB" added.'
		WRITE (6,*) '            If not given, output goes to T'
     &		 //'MP.SDB'
		WRITE (6,*)
		WRITE (6,*) 'Mixture assignments are read from SPECASN.D'
     &		 //'AT.'
		WRITE (6,*) 'SAROAD assignments are read from [SARtype]A'
     &		 //'SN.DAT'
		WRITE (6,*) 'These files need to be in the local directory'
		CALL EXIT (2)
	ELSEIF (SARTYP(1:1).EQ.' ') THEN
		WRITE (6,*) 'Need to specify SAROAD type with '//
     &		 INFILE(1:LNBLNK(INFILE))
		CALL EXIT (2)
	ENDIF
	LINFIL=LNBLNK(INFILE)
C
	IF (OUTFIL(1:1).EQ.' ') THEN
		OUTFIL='TMP.SDB'
		LOUTFL=7
	ELSE
		LOUTFL=LNBLNK(OUTFIL)
 		IF (OUTFIL(LOUTFL-3:LOUTFL).NE.'.SDB') THEN
 			OUTFIL=OUTFIL(1:LOUTFL)//'.SDB'
 			LOUTFL=LOUTFL+4
 		ENDIF
	ENDIF
C
C
C	READ ASSIGNMENT DATA USED
C
C	READ ID-TO=COMPOUND ASSIGNMENTS FROM SPECASN.DAT
C
	OPEN (UNIT=2,FILE='SPECASN.DAT',STATUS='OLD',ERR=902)
	READ (2,1,END=903) INLIN
    1	FORMAT (A)
  21	READ (2,1,END=903) INLIN
	IF (INLIN(1:1).NE.'.') GOTO 21
	NSPCID=0
	NUID=0
	OLDUID=" "
  22	READ (2,1,END=28) INLIN
	IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') GOTO 22
	IF (NSPCID.EQ.MAXSPC) THEN
		WRITE (6,*) 'TOO MANY UID-TO-COMPOUND ASSIGNMENTS'
		WRITE (6,*) 'INCREASE MAXSPC IN PROFPRE.FOR'
		GOTO 904
	ENDIF
	NSPCID=NSPCID+1
	READ (INLIN,23,ERR=904) UID,UIDSPC(NSPCID),
     &	 SPCWT(NSPCID)
   23	FORMAT(2A16,F10.8)
	IF (UID.NE.OLDUID) THEN
		IF (NUID.GT.0) NUID2(NUID)=NSPCID-1
		IF (NUID.EQ.MAXUID) THEN
			WRITE (6,*) 'TOO MANY UID''S.'
			WRITE (6,*) 'INCREASE MAXUID IN PROFPROM.FOR'
			GOTO 904
		ENDIF
		NUID=NUID+1
		UIDLST(NUID)=UID
		NUID1(NUID)=NSPCID
		OLDUID=UID
	ENDIF
	GOTO 22
   28	IF (NUID.GT.0) NUID2(NUID)=NSPCID
	CLOSE (UNIT=2)
C
C	READ SAROAD ASSIGNMENTS FROM [SARID]ASN.DAT
C
	IF (SARTYP.EQ.'NONE'.OR.SARTYP.EQ.'SDB ') THEN
C
C		SARTYPE="NONE" MEANS "SAROAD" ID'S ARE
C		ACTUALLY UNIQUE ID'S
C
		NOSAR=.TRUE.
	ELSE
		NOSAR=.FALSE.
		LSARTP=LNBLNK(SARTYP)
		INLIN=SARTYP(1:LSARTP)//'SARID.DAT'
		OPEN (UNIT=2,FILE=INLIN,STATUS='OLD',ERR=905)
		READ (2,1,END=905) INLIN
 121		READ (2,1,END=906) INLIN
		IF (INLIN(1:1).NE.'.') GOTO 121
		NSARID=0
 122		READ (2,1,END=128) INLIN
		IF (INLIN(1:1).EQ.' '.OR.INLIN(1:1).EQ.'!') 
     &		 GOTO 122
		IF (NSARID.EQ.MAXSAR) THEN
  			WRITE (6,*) 'TOO MANY SAROAD '
     &			 //'ASSIGNMENTS'
  			WRITE (6,*) 'INCREASE MAXSAR IN '
     &			 //'EMITOEMC.FOR'
			GOTO 906
		ENDIF
		CALL PARSE0 (2,INLIN,80)
		NSARID=NSARID+1
		CALL PARSE1 (1,SARNAM(NSARID),I,16,INLIN)
		CALL PARSE1 (2,SARID(NSARID),I,16,INLIN)
		IF (SARID(NSARID)(1:1).EQ.' ') NSARID=NSARID-1
		GOTO 122
  128		CLOSE (UNIT=2)
	ENDIF
C
C	OPEN INPUT AND OUTPUT FILES
C
	OPEN (UNIT=1,FILE=INFILE,STATUS='OLD',ERR=111)
	GOTO 112
  111	WRITE (6,*) 'File = '//INFILE(1:LINFIL)//
     &	 ' does not exist'
	CALL EXIT (2)
C
  112	OPEN (UNIT=2,FILE=OUTFIL,STATUS='UNKNOWN')
	WRITE (2,1) 'MULTI-PROF Input='//INFILE(1:LINFIL)//' '//SARTYP
C
	PROFIN=' '
	EOF=.FALSE.
	NPROFS=0
C
   99	READ (1,1,END=180) INLIN
	CALL PARSE0 (3,INLIN,80)
	CALL PARSE1 (1,PROFIN,I,16,INLIN)
	IF (PROFIN(1:1).EQ.' ') GOTO 99
C
C	NEW PROFILE -- FROM LAST LINE READ
C
  100	NCMP=0
	TOTWT=0.0
	PROFID=PROFIN
	GOTO 131
C
C	READ NEXT LINE OF INPUT FILE AND PROCESS
C
  130	READ (1,1,END=180) INLIN
	IF (INLIN(1:1).EQ.'!') GOTO 130
	CALL PARSE0 (3,INLIN,80)
	CALL PARSE1 (1,PROFIN,I,16,INLIN)
	IF (PROFIN.EQ.' ') GOTO 130
	IF (PROFIN.NE.PROFID) GOTO 181
C
  131	CALL PARSE1 (2,SARI,I,16,INLIN)
	CALL PARSER (3,WTI,ERRIND,INLIN)
	IF (ERRIND.OR.SARI(1:1).EQ.' ') GOTO 130
	TOTWT=TOTWT+WTI
C
C	GET UNIQUE ID
C
	IF (NOSAR) THEN
		UID=SARI
	ELSE
		DO 132 I=1,NSARID
		IF (SARNAM(I).EQ.SARI) THEN
			UID=SARID(I)
			GOTO 134
		ENDIF
  132		CONTINUE
		WRITE (6,*) 'No assignment for SAROAD ',SARI
		GOTO 999
	ENDIF
C
  134	CONTINUE
C
C	GET COMPOUND ASSIGNMENTS FOR UNIQUE ID UID.
C
	DO 140 I=1,NUID
	IF (UID.EQ.UIDLST(I)) THEN
		DO 142 J=NUID1(I),NUID2(I)
			CMPI=UIDSPC(J)
			WTJ=WTI*SPCWT(J)
			IF (NCMP.GT.0) THEN
				DO 144 K=1,NCMP
				IF (CMPI.EQ.CMPID(K)) THEN
					CMPWT(K)=CMPWT(K)+WTJ
 					GOTO 142
 				ENDIF
  144				CONTINUE
			ENDIF
			IF (NCMP.EQ.MAXCMP) THEN
				WRITE (6,*) 
     &				 'TOO MANY COMPOUNDS.'
				WRITE (6,*)
     &				 'INCREASE MAXCMP IN '
     &				 //'EMITOEMC.FOR'
				GOTO 906
			ENDIF
			NCMP=NCMP+1
			CMPID(NCMP)=CMPI
			CMPWT(NCMP)=WTJ
  142		CONTINUE
		GOTO 130
	ENDIF
  140	CONTINUE
	WRITE (6,*) 'No compound assignment for UID ',UID
	GOTO 999
C
C	NORMAL END OF EMI FILE INPUT.
C
  180	EOF=.TRUE.
  181	IF (TOTWT.EQ.0.0) GOTO 190
	TOTWT2=0.0
	UNAWT=0.0
	UNVWT=0.0
	DO 182 I=1,NCMP
	WTI=CMPWT(I)
	IF (CMPID(I)(1:6).EQ.'NOASN ') THEN
		UNAWT=UNAWT+WTI
	ELSEIF (CMPID(I)(1:7).EQ.'NONVOL ') THEN
		UNVWT=UNVWT+WTI
	ENDIF
  182	TOTWT2=TOTWT2+WTI
C
	WRITE (2,1) '..'
	WRITE (2,1) PROFID(1:LNBLNK(PROFID))
	DO 185 I=1,NCMP
	IF (CMPWT(I).NE.0.0) WRITE (2,186) CMPID(I),CMPWT(I)
  185	CONTINUE
  186	FORMAT (A16,1PE10.4)
	NPROFS=NPROFS+1
C
  190	IF (EOF) GOTO 800
	GOTO 100
C
C	ERROR PROCESSING DATA FOR THE INPUT .EMI FILE
C
C
C	END OF INPUT.
C
  800	CLOSE (UNIT=1)
	CLOSE (UNIT=2)
	CLOSE (UNIT=3)
	WRITE (6,801) NPROFS,INFILE(1:LINFIL),SARTYP
  801	FORMAT (' ProfPre processed ',I5,' profiles from ',A,' for ',A)
	IF (OUTFIL(1:LOUTFL).NE.'TMP.SDB') WRITE (6,*)
     &	 'File = '//OUTFIL(1:LOUTFL)//' output.'
	CALL EXIT (0)
C
C	ERRORS
C
 902	WRITE (6,*) 'Error opening SPECASN.DAT for compound assignments'
	GOTO 999
 903	WRITE (6,*) 'No data in a compound assignment file.'
	CLOSE (UNIT=2)
	GOTO 999
 904	WRITE (6,*) 'Error processing data in compound assignment file.'
	WRITE (6,*) 'Bad record is: ',INLIN
	CLOSE (UNIT=2)
	GOTO 999
 905	WRITE (6,*) 'Error opening SAROAD assignment file = '
     &	 //INLIN(1:LNBLNK(INLIN))
	CLOSE (UNIT=3)
	CLOSE (UNIT=4,STATUS='DELETE')
	GOTO 999
 906	WRITE (6,*) 'No data or error in SAROAD assignment file = '
     &	 //SARTYP(1:LSARTP)//'SARID.DAT'
 999	CALL EXIT (2)
	END
